--[[
Ghost.lua v0.55
a script by Vlad Tepes
]]

ROUND_TIME_LIMIT	= 2 * 60	-- Round time limit in sec
ROUND_ENDTIME		= 5			-- Number of seconds before restart when round ended

MAX_GHOST_ROUNDS	= 3			-- Number of maximum consecutive ghost round


MSG_YOUARETHEGHOST	= "You are the ghost! Press mic key to jump, press mic when close to a wall to cling"
MSG_XXXISTHEGHOST	= " is the ghost! Go and hunt it"

FLAMETHROWER		= true		-- also gives a flamethrower with a few ammo to marines





--[[ DON'T MODIFY BEYOND THIS LINE IF YOU DON'T KNOW WHAT YOU ARE DOOING ]]--

JUMP_ACCELERATION	= 0.2
JUMP_COOLDOWN		= 1.0 -- Minimum time (in sec) between two jumps

PLAYER_RADIUS		= 0.2
PLAYER_HEIGHT		= 0.8

PLAYER_GUN_HEIGHT	= 0.6
CLING_TOLERANCE_H	= 0.3
CLING_TOLERANCE_V	= 0.3

--[[ VARIABLES INITIALIZATION ]]--

timer_round		= 0
timer_restart	= 30 * ROUND_ENDTIME
round_over		= false
ghost_won_rounds = MAX_GHOST_ROUNDS

chooser_pointscount = 0


--[[ TRIGGERS ]]--

Triggers = {}

function Triggers.init()
	cleanLevel()

	-- initialize and populate custom fields
	for p in Players() do
		p._ghost	= false
		p._jumpcd	= 0 --number of ticks before being able to make another jump
		p._cling	= false
      	p._x		= p.x
      	p._y		= p.y
      	p._x		= p.z
      	p._poly		= p.polygon
      	p._gcpoints = 1
	end
end

function Triggers.idle()
	if Game.type ~= "netscript" then
		Players.print("This game is supposed to be played with netscript gametype !")
		if timer_restart == 0 then Game.over = true end
		if timer_restart > 0 then timer_restart = timer_restart -1 end
	else

		for p in Players() do
				if p.dead then
					p.action_flags.action_trigger = false
				else
					if p._ghost then
						p.invisibility_duration = 9999 --ghost is invisible
						manageJump(p)
					end
				end
				p.overlays[0].text = "Time remaining : ".. math.ceil(timer_round/30) .."sec"
		end
	
		if Game.ticks == 0 then
			startNewRound()
			for p in Players() do
				setStuff(p)
			end
		end
		
		if round_over and timer_restart == 0 then
			startNewRound()
			round_over = false
		end
		
	
		if not round_over then
			local rO, gw = checkIfRoundOver()
			if rO then --round over
				round_over = true
				timer_restart = 30 * ROUND_ENDTIME
				if gw then
					ghost_won_rounds = ghost_won_rounds + 1
					Players.print("The ghost wins the round!")
				else
					ghost_won_rounds = MAX_GHOST_ROUNDS -- Force ghost change
					Players.print("The ghost has been killed!")
				end
				for p in Players() do
					p:damage(1000)
					
					if not p._ghost and ghost_won_rounds == MAX_GHOST_ROUNDS then
						local percents = math.floor ( 100* p._gcpoints / chooser_pointscount ) 
						p:print("You have ".. percents .."% to be the next ghost")
					end
				end
			end
		
		end
		
		if round_over and timer_restart > 0 then timer_restart = timer_restart - 1 end
		if timer_round > 0 then timer_round = timer_round - 1 end
	
	end
end

function Triggers.player_revived(player)
		setStuff(player)
end

function Triggers.player_damaged(victim, aggressor_player, aggressor_monster, damage_type, damage_amount, projectile)
	if aggressor_player ~= nil then
		if victim._ghost and not aggressor_player._ghost then
			aggressor_player._gcpoints = aggressor_player._gcpoints + damage_amount
			chooser_pointscount = chooser_pointscount + damage_amount
		end
	end
end

function Triggers.player_killed(player, aggressor_player, action, projectile)
	if aggressor_player ~= nil and aggressor_player ~= victim then --Check if real kill
		if player._ghost ~= aggressor_player._ghost then
			aggressor_player.points = aggressor_player.points + 1
		end
	end
end


--[[ GAME MANAGEMENT AREA ]]--

function startNewRound()
	if ghost_won_rounds == MAX_GHOST_ROUNDS then
		local ghost = chooseGhost()
		-- reset custom fields
		for p in Players() do
			p._ghost = (p == ghost)
		end
	
		Players.print(ghost.name .. MSG_XXXISTHEGHOST)
		
	end
	
	for p in Players() do
		p._gcpoints = 1
		p.action_flags.action_trigger = true
	end
	chooser_pointscount = #Players
	timer_round = ROUND_TIME_LIMIT * 30
end

function chooseGhost()
	-- calculate total number of points
	local ghostChooserPoints = 0
	for p in Players() do
		ghostChooserPoints = ghostChooserPoints + p._gcpoints
	end
	
	local i = Game.random(ghostChooserPoints)
	ghostChooserPoints = 0
	for p in Players() do
		ghostChooserPoints = ghostChooserPoints + p._gcpoints
		if i < ghostChooserPoints then return p end
	end
end

-- Check if round is over and give the wining side,
-- the first boolean of the couple is if round over
-- the second boolean is if ghost won
function checkIfRoundOver()
	-- count the number of ghost and non ghost alive...
	local nGhost, nOthers = 0, 0
	for p in Players() do
		if not p.dead then
			if p._ghost then
				nGhost = nGhost + 1
			else
				nOthers = nOthers + 1
			end
		end
	end
	if nOthers == 0 then
		return true, true
	elseif nGhost == 0 or timer_round == 0 then
		return true, false
	else
		return false, false
	end
end


--[[ GHOST MOVEMENTS AREA ]]--

--Players = {}
--function Players:_jump()
--	self:accelerate(self.direction, JUMP_ACCELERATION*math.cos(math.rad(self.elevation)), JUMP_ACCELERATION*math.sin(math.rad(self.elevation)))
--end

function manageJump(player)
	if (player.external_velocity.z == 0 or player._cling) and player._jumpcd <= 0 then
		if player.action_flags.microphone_button then
			player:accelerate(player.direction, JUMP_ACCELERATION*math.cos(math.rad(player.elevation)), JUMP_ACCELERATION*math.sin(math.rad(player.elevation)))
--			player:jump()
			player.monster:play_sound("bob hit")
			player._jumpcd = 30 * JUMP_COOLDOWN
			player._cling = false
		end	
	end
	
	
	if hangAllowed(player) and player.action_flags.microphone_button then
		player._cling = true
      	player._x = player.x
      	player._y = player.y
      	player._z = player.z
      	player._poly = player.polygon
	end
	
	if player._cling then
		player:position(player._x, player._y, player._z, player._poly)
		player.external_velocity.x = 0
		player.external_velocity.y = 0
		player.external_velocity.z = 0
	end
	
	
	player.action_flags.microphone_button = false
	if player._jumpcd > 0 then player._jumpcd = player._jumpcd - 1 end
end

function hangAllowed(player)
	local t, x, y, z, poly = player:find_target()
	local distanceH, distanceV
	
	distanceH = math.sqrt((x-player.x)^2 + (y-player.y)^2)
	distanceV = math.abs(z - player.z - PLAYER_GUN_HEIGHT)
	
	if is_side(t) then
		return (distanceH <= CLING_TOLERANCE_H)
	end
	
	if is_polygon_ceiling(t) then
		return (distanceV <= CLING_TOLERANCE_V and distanceH <= 2*CLING_TOLERANCE_H)
	end
end


--[[ ITEM MANAGEMENT AREA ]]--

function setStuff(player)
	if player._ghost then
		setGhostStuff(player)
	else
		setOtherStuff(player)
	end
end

function setGhostStuff(player)
	clearItems(player)
	player.items["knife"] = 1
	player.weapons["fist"]:select()
	player:print(MSG_YOUARETHEGHOST)
end

function setOtherStuff(player)
	clearItems(player)
	player.items["smg"] = 1
	player.items["smg ammo"] = 4
	player.items["pistol"] = 1
	player.items["pistol ammo"] = 4
	
	if FLAMETHROWER then
		player.items["flamethrower"] = 1
		player.items["flamethrower ammo"] = 1
	end
	
	player.weapons["smg"]:select()
end

function clearItems(player)
	for itemType in ItemTypes() do
		player.items[itemType] = 0
	end
end

function cleanLevel()
	for item in Items() do 
		item:delete()
	end

	function Triggers.item_created(item)
		item:delete()
	end
end